package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.activiti.service.ActivitiService;
import com.qmth.sop.business.bean.result.TBQualityProblemApplyResult;
import com.qmth.sop.business.entity.TBQualityProblemApply;
import com.qmth.sop.business.mapper.TBQualityProblemApplyMapper;
import com.qmth.sop.business.service.SequenceService;
import com.qmth.sop.business.service.TBQualityProblemApplyService;
import com.qmth.sop.business.service.TFCustomFlowEntityService;
import com.qmth.sop.common.enums.InfluenceDegreeEnum;
import com.qmth.sop.common.enums.QualityProblemReasonEnum;
import com.qmth.sop.common.enums.QualityProblemTypeEnum;
import com.qmth.sop.common.enums.TFCustomTypeEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

/**
 * 质量问题反馈表 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-14 14:00:01
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class TBQualityProblemApplyServiceImpl extends ServiceImpl<TBQualityProblemApplyMapper, TBQualityProblemApply> implements TBQualityProblemApplyService {

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    @Resource
    ActivitiService activitiService;

    @Resource
    SequenceService sequenceService;

    /**
     * 查询列表
     *
     * @param iPage
     * @return
     */
    @Override
    public IPage<TBQualityProblemApplyResult> query(IPage<Map> iPage, Long serviceId, Long userId, QualityProblemTypeEnum type, QualityProblemReasonEnum reason, InfluenceDegreeEnum degree, String custom, String problemNo, Long startTime, Long endTime) {
        return this.baseMapper.query(iPage, serviceId, userId, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(reason) ? reason.name() : null, Objects.nonNull(degree) ? degree.name() : null, custom, problemNo, startTime, endTime);
    }

    /**
     * 新增修改质量问题反馈表
     *
     * @param tbQualityProblemApply
     */
    @Override
    @Transactional
    public Boolean saveTBQualityProblemApply(TBQualityProblemApply tbQualityProblemApply) throws InterruptedException {
        HttpServletRequest request = ServletUtil.getRequest();
        Map<String, Object> map = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        map.putAll(formDataMap);
        Map<String, Object> resultMap = activitiService.taskApprove(tbQualityProblemApply.getFlowDeploymentId(), tbQualityProblemApply.getApprove(), null, null, tbQualityProblemApply.getFormProperties(), tbQualityProblemApply.getApproveUserIds(), null, map);
        String problemNo = sequenceService.createFlowCode(TFCustomTypeEnum.QUALITY_PROBLEM_FLOW);
        tbQualityProblemApply.setProblemNo(problemNo);
        return this.saveOrUpdate(tbQualityProblemApply);
    }

    /**
     * 删除质量问题反馈表
     *
     * @param ids
     */
    @Override
    @Transactional
    public Boolean delete(long[] ids) {
        for (long id : ids) {
            this.removeById(id);
        }
        return true;
    }

}